import json
import pandas as pd

def ontology_to_dataframe(json_obj, file_name=None):
    rows = []
    row = []
    for category, cat_lst in json_obj.items():
        print(f"{category=}")
        print(f"{cat_lst=}")
        row = [category]
        if isinstance(cat_lst, dict):
            row.append(cat_lst['id'])
            row.append(cat_lst['name'])
            rows.append(row)
            row = [category]
        else:
            for cat_lst_item in cat_lst:
                print(f"{cat_lst_item=}")
                row.append(cat_lst_item['id'])
                row.append(cat_lst_item['name'])
                rows.append(row)
                row = [category]

    df = pd.DataFrame(rows)
    df.columns=["Category","Name","Id"]
    if file_name:
        df.to_csv(file_name, index=False)
    return df


def refereneces_to_dataframe(json_obj, file_name=None):
    df = pd.DataFrame(json_obj)
    #df.columns=["Category","Name","Id"]
    if file_name:
        df.to_csv(file_name, index=False)
    return df




import requests
from bs4 import BeautifulSoup
from datetime import datetime, timedelta
import urllib.parse

def biorxiv_most_recent_30__posts():
    url = "https://connect.biorxiv.org/biorxiv_xml.php?subject=all"
    response = requests.get(url)
    if response.status_code == 200:
        return response.content
    else:
        raise Exception(f"Failed to retrieve data: {response.status_code}")

# Function to search for keywords in text
def _contains_keywords(text, keywords):
    return any(keyword.lower() in text.lower() for keyword in keywords)


def biorxiv_recent_posts_filtered(keywords = ["caenorhabditis", "elegans"]):
    xml_data = biorxiv_most_recent_30__posts()
    soup = BeautifulSoup(xml_data, "xml")
    articles = []

    for item in soup.find_all('item'):
        title = item.find('title').get_text(strip=True)
        description = item.find('description').get_text(strip=True)
        dc_date = item.find('dc:date').get_text(strip=True)
        dc_identifier = item.find('dc:identifier').get_text(strip=True)
        
        # Check if either the title or description contains any of the keywords
        if _contains_keywords(title, keywords) or _contains_keywords(description, keywords):
            # Append the article details as a dictionary
            articles.append({
                'title': title,
                'date': dc_date,
                'doi': f"https://doi.org/{dc_identifier}"
            })

    return articles


def biorxiv_search(search_criteria="caenorhabditis elegans", days=1):
    result_dict = []  # Initialize an empty list to return

    try:
        url = encode_biorxiv_search(search_criteria, days)
        response = requests.get(url)

        if response.status_code == 200:
            soup = BeautifulSoup(response.text, 'html.parser')

            search_results_div = soup.find('div', class_='panel-pane pane-highwire-search-results active', id='hw-advance-search-result')

            if search_results_div:
                for li in search_results_div.find_all('li'):
                    a_tag = li.find('a', class_='highwire-cite-linked-title')
                    if a_tag:
                        # Extract the href and title
                        url = a_tag['href']
                        title = a_tag.find('span', class_='highwire-cite-title').text

                        # Create a dictionary for each entry
                        result_dict.append({'url': f"https://www.biorxiv.org{url}", 'title': title})
            else:
                print("Specified div not found.")
        else:
            print(f"Request failed with status code {response.status_code}")

    except Exception as e:
        print(f"An error occurred: {e}")

    return result_dict  # Return the result (could be empty if there was an error)

def encode_biorxiv_search(search_criteria, days=1):
    base_url = "https://www.biorxiv.org/search/"
    today = datetime.today()
    start_date = (today - timedelta(days=days)).strftime('%Y-%m-%d')
    end_date = today.strftime('%Y-%m-%d')

    # Construct the query string with placeholders
    query_string = f"abstract_title:{search_criteria} abstract_title_flags:match-any jcode:biorxiv limit_from:{start_date} limit_to:{end_date} numresults:75 sort:publication-date direction:ascending format_result:condensed"
    
    # Encode the query string
    encoded_query = urllib.parse.quote(query_string)
    
    # Combine the base URL with the encoded query string
    full_url = base_url + encoded_query
    return full_url




import pytest
from pub_worm.impact_factor.impact_factor_lookup import get_impact_factor


def test_get_impact_factor_existing_issn():
    # Setup: Mock the impact_factor_df DataFrame to return a sample row
    issn = '0007-9235'
    expected_result = 286.13
    actual_result = get_impact_factor(issn)
    print(actual_result)
    # Assertion: Check that the function returns the expected result
    assert actual_result == expected_result

def test_get_impact_factor_non_existing_issn():
    # Setup: Mock the impact_factor_df DataFrame to be empty
    issn = '1234-5678'

    # Action: Call the function
    result = get_impact_factor(issn)

    # Assertion: Check that the function returns None for non-existing ISSN
    assert result is None

if __name__ == "__main__":
    pytest.main([__file__])



import os
import requests
import gzip
import shutil
import csv
import json
from bs4 import BeautifulSoup
import pandas as pd

# Get the most current Wormbase DB
def current_wormbase_version():
    api_url = f'http://rest.wormbase.org//rest/database/version'
    # Absolutley no error checking is done!!
    response = requests.get(api_url)
    json_data = json.loads(response.text)
    if  response.status_code == 200:
        return json_data['data']
    else:
        return {'error':'something is not right'}



def annotation_files_list(wormbase_version):
    url = f"https://downloads.wormbase.org/releases/{wormbase_version}/species/c_elegans/PRJNA13758/annotation/"
    
    response = requests.get(url, timeout=10)
    
    # Check if the request was successful
    if response.status_code != 200:
        print(f"Failed to retrieve webpage. Status code: {response.status_code}")
        return []
    
    # Parse HTML content using Beautiful Soup
    soup = BeautifulSoup(response.content, 'html.parser')
    
    # Find all table rows
    rows = soup.find_all('tr')
    
    # Initialize list to store file names
    file_names = []
    
    # Iterate over table rows
    for row in rows:
        # Find the second table data (td) element in the row
        td = row.find_all('td')
        
        # Check if the row has at least two td elements
        if len(td) > 1:
            file_name = td[1].find('a').text
            file_names.append(file_name)
    
    # Remove the first element (Parent Directory)
    file_names = file_names[1:]
    
    prefix = f"c_elegans.PRJNA13758.{wormbase_version}."
    file_names = [name.replace(prefix, "") for name in file_names]
    
    return file_names

   
def _download_url(file_url, output_file_path):
    response = requests.get(file_url, stream=True)
    if response.status_code == 200:
        with open(output_file_path, 'wb') as f:
            shutil.copyfileobj(response.raw, f)
        print(f"Downloaded: {output_file_path}")
    else:
        print(f"Failed to download: {file_url} (status code: {response.status_code})")
    return

def download_annotation_file(wormbase_version, file_nm, output_dir):
    annotation_nm = f"c_elegans.PRJNA13758.{wormbase_version}.{file_nm}"

    base_url = f"https://downloads.wormbase.org/releases/{wormbase_version}/species/c_elegans/PRJNA13758"
    file_url = f"{base_url}/annotation/{annotation_nm}"

    # Ensure output directory exists
    os.makedirs(output_dir, exist_ok=True)

    # Download the file
    output_file_path = os.path.join(output_dir, annotation_nm)
    _download_url(file_url, output_file_path)

    ext_nm = annotation_nm[-3:]
    if ext_nm == ".gz":
        # Unzip the file
        with gzip.open(output_file_path, 'rb') as f_in:
            with open(output_file_path.rstrip('.gz'), 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)

        # Remove the .gz file if it exists
        if os.path.exists(output_file_path):
            os.remove(output_file_path)
            
        print(f"Unzipped: {output_file_path}")
    
def download_gene_ids(wormbase_version, output_dir):
        return download_annotation_file(wormbase_version, "geneIDs.txt.gz", output_dir)
    
        
def extract_live_gene_ids(wormbase_version, source_dir):
    gene_ids = f"c_elegans.PRJNA13758.{wormbase_version}.geneIDs.txt"
    input_file = f"{source_dir}/{gene_ids}"

    if not os.path.exists(input_file):
        print(f"File '{input_file}' does not exist.")
        return
    
    # Generate the output file name
    output_file = f"{input_file[:-3]}csv"

    # Load the input CSV file into a DataFrame
    df = pd.read_csv(input_file, header=None)

    # Filter rows where the 5th column equals 'Live'
    df_filtered = df[df[4] == 'Live']

    # Select the required columns (2nd, 3rd, 4th, and 6th)
    df_selected = df_filtered[[1, 2, 3, 5]]

    # Add the appropriate column headers
    df_selected.columns = ["Wormbase_Id", "Gene_name", "Sequence_id", "Gene_Type"]

    # Save the result to the output file
    df_selected.to_csv(output_file, index=False)

    print(f"Processed file saved to: {output_file}")
 


import requests
import aiohttp
import asyncio
import aiofiles

def get_sequence_region(start_position, end_position, chromosome, species = "caenorhabditis_elegans"):
    sequence = ""
    try:
        # Construct the URL for the Ensembl API
        url = f"https://rest.ensembl.org/sequence/region/{species}/{chromosome}:{start_position}..{end_position}?content-type=text/plain"
        
        # Send the GET request
        response = requests.get(url, timeout=10)

        # Check if the request was successful
        if response.status_code == 200:
            sequence = response.text
        else:
            print(f"Failed to retrieve sequence. Status code: {response.status_code}")

    except requests.exceptions.Timeout:
        print("The request timed out. Please try again later.")
    except requests.exceptions.RequestException as e:
        print(f"An error occurred: {e}")
    return sequence


async def async_get_sequence_region(start_position, end_position, chromosome, species="caenorhabditis_elegans", max_retries=3):
    sequence = ""
    url = f"https://rest.ensembl.org/sequence/region/{species}/{chromosome}:{start_position}..{end_position}?content-type=text/plain"
    
    retries = 0

    while retries < max_retries:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, timeout=10) as response:
                    # Check if the request was successful
                    if response.status == 200:
                        sequence = await response.text()
                        return sequence  # Return if successful
                    elif response.status_code == 429:
                        retry_after = response.headers.get('Retry-After')
                        if retry_after:
                            print(f"Rate limited. Retrying after {retry_after} seconds...")
                            asyncio.sleep(float(retry_after))  # Wait for the time specified by the server
                            retries += 1  # Increment retry count
                        else:
                            print("Rate limited. No Retry-After Waiting 2 seconds...")
                            asyncio.sleep(2)  # Wait for the time specified by the server
                            retries += 1  # Increment retry count
                            
                    else:
                        print(f"Failed to retrieve sequence. Status code: {response.status}")
                        retries += 1
                        await asyncio.sleep(2)  # Wait before retrying

        except Exception as e:
            print(f"An error occurred: {e} Retry {retries + 1}/{max_retries}.")
            retries += 1
            await asyncio.sleep(2)  # Wait before retrying

    print(f"Failed to retrieve sequence after {max_retries} attempts.")
    return sequence  # Return empty sequence if retries exhausted



def create_fasta(gene_nm, chromosome, sequence, path="."):
    # Define the filename based on the gene name
    filename = f"{path}/{gene_nm}.fasta"
    
    # Open the file in write mode
    with open(filename, 'w') as file:
        # Write the header line
        file.write(f">{chromosome}\n")
        
        # Write the sequence in lines of 60 characters
        for i in range(0, len(sequence), 60):
            file.write(sequence[i:i+60] + "\n")


async def async_create_fasta(gene_nm, chromosome, sequence, path="."):
    # Define the filename based on the gene name
    filename = f"{path}/{gene_nm}.fasta"
    
    # Open the file in asynchronous write mode
    async with aiofiles.open(filename, 'w') as file:
        # Write the header line
        await file.write(f">{chromosome}\n")
        
        # Write the sequence in lines of 60 characters
        for i in range(0, len(sequence), 60):
            await file.write(sequence[i:i+60] + "\n")
    
    


import pytest
import inspect
import os
import json
from bs4 import BeautifulSoup
from pub_worm.ncbi.entreze_api import EntrezAPI

DUMP_API_CALL = True
def dump_api_call(function_name, actual_result,data_type="json"):
    if DUMP_API_CALL:
        output_path = "output"
        if not os.path.exists(output_path):
            os.makedirs(output_path)

        if data_type == 'json':
            pretty_data = json.dumps(actual_result, indent=4)
        elif data_type == 'json':
            soup = BeautifulSoup(actual_result, "xml")
            # Pretty-print the XML content
            pretty_data = soup.prettify()
        else:
            data_type=None

        if data_type:
            with open(f"{output_path}/{function_name}.{data_type}", 'w') as file:
                file.write(pretty_data)

        #with open("output.txt", "w") as file:
        #file.write(mystring)


def test_entreze_esearch():
    function_name = inspect.currentframe().f_code.co_name
    search_term = "Marnett L[au] AND (2019/01/01:2024/04/16[pdat])AND (vanderbilt[affil])"
    esearch_params = {'term': search_term }

    ncbi_api = EntrezAPI()
    actual_result = ncbi_api.entreze_esearch(esearch_params)
    dump_api_call(function_name, actual_result, "json")

    expected_result = "1"
    assert 'query_key' in actual_result, "'query_key' found in result"
    assert actual_result['query_key'] == expected_result


def test_entreze_epost():
    function_name = inspect.currentframe().f_code.co_name
    data = ["10021351", "10022905", "10022914", "10022975", "10048487", "10048958", "10049162", "10049362", "10049567", "10049576", "10051671", "10051850", "10064800", "10066248"]

    ncbi_api = EntrezAPI()
    actual_result = ncbi_api.entreze_epost(data)
    dump_api_call(function_name, actual_result, "json")

    expected_result = "1"
    assert 'query_key' in actual_result, "'query_key' found in result"
    assert actual_result['query_key'] == expected_result


def test_entreze_pmid_summaries():
    function_name = inspect.currentframe().f_code.co_name
    data = ["10021351"]

    ncbi_api = EntrezAPI()
    interm_result = ncbi_api.entreze_epost(data)

    actual_result = {}
    if 'WebEnv' in interm_result:
        actual_result = ncbi_api.entreze_pmid_summaries(interm_result)
        dump_api_call(function_name, actual_result, "json")


    assert len(actual_result) == 1, f"Expected exactly 1 result found but {len(actual_result)}"
    assert 'issn' in actual_result[0], "'issn' not found in result"
    assert actual_result[0]['issn'] == "0950-1991",f"Expected 'issn' to equal 0950-1991 but found {actual_result[0]['issn']}"

def test_entreze_epost_efetch():
    function_name = inspect.currentframe().f_code.co_name
    data = ["10021351"]

    ncbi_api = EntrezAPI()
    interm_result = ncbi_api.entreze_epost(data)

    actual_result = {}
    assert 'WebEnv' in interm_result, "Expected 'WebEnv' in interm_result"

    actual_result = ncbi_api.entreze_efetch(interm_result)
    dump_api_call(function_name, actual_result, "json")

    assert len(actual_result['articles']) == 1, f"Expected exactly 1 result found but {len(actual_result['articles'])}"
    assert 'pub_abbr' in actual_result['articles'][0], "'pub_abbr' not found in result"
    assert actual_result['articles'][0]['pub_abbr'] == "Development",f"Expected 'pub_abbr' to equal Development but found {actual_result['articles'][0]['pub_abbr']}"

def test_entreze_epost_efetch_full_article():
    function_name = inspect.currentframe().f_code.co_name
    data = ["PMC8253488","6358295"]
    params= {'db': 'pmc'}

    ncbi_api = EntrezAPI()
    interm_result = ncbi_api.entreze_epost(data, params)

    actual_result = {}
    assert 'WebEnv' in interm_result, "Expected 'WebEnv' in interm_result"

    actual_result = ncbi_api.entreze_efetch(interm_result)
    dump_api_call(function_name, actual_result, "json")

    #assert len(actual_result) == 1, f"Expected exactly 1 result found but {len(actual_result)}"
    #assert 'pub_abbr' in actual_result[0], "'pub_abbr' not found in result"
    #assert actual_result[0]['pub_abbr'] == "Development",f"Expected 'pub_abbr' to equal Development but found {actual_result[0]['pub_abbr']}"

def test_entreze_esearch_to_elink():
    function_name = inspect.currentframe().f_code.co_name
    search_term = "Marnett L[au] AND (2019/01/01:2024/04/16[pdat])AND (vanderbilt[affil])"
    esearch_params = {'term': search_term }

    ncbi_api = EntrezAPI()
    interm_result = ncbi_api.entreze_esearch(esearch_params)

    expected_result = "1"
    assert 'query_key' in interm_result, "'query_key' found in result"
    assert interm_result['query_key'] == expected_result

    actual_result = ncbi_api.entreze_elink_pmid_to_pmcid(interm_result)
    dump_api_call(function_name, actual_result, "json")

def test_entreze_esearch_efetch():
    function_name = inspect.currentframe().f_code.co_name
    
    search_term = "Marnett L[au] AND (2019/01/01:2024/04/16[pdat])AND (vanderbilt[affil])"
    esearch_params = {'term': search_term }

    ncbi_api = EntrezAPI()
    interm_result = ncbi_api.entreze_esearch(esearch_params)

    actual_result = {}
    assert 'WebEnv' in interm_result, "Expected 'WebEnv' in interm_result"

    actual_result = ncbi_api.entreze_efetch(interm_result)
    dump_api_call(function_name, actual_result, "json")

    # assert len(actual_result) == 1, f"Expected exactly 1 result found but {len(actual_result)}"
    # assert 'issn' in actual_result[0], "'issn' not found in result"
    # assert actual_result[0]['issn'] == "0950-1991",f"Expected 'issn' to equal 0950-1991 but found {actual_result[0]['issn']}"


if __name__ == "__main__":
    pytest.main([__file__])



"""
Setup for pypi releases of pub_worm
"""
from pub_worm import __version__
from setuptools import setup, find_packages
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='pub_worm',
      version=__version__,
      description='Wormbase/PudMed API Access',
      long_description_content_type="text/markdown",
      long_description=long_description,

      url='https://github.com/DanHUMassMed/pub_worm.git',
      author='Dan Higgins',
      author_email='daniel.higgins@yahoo.com',
      license='MIT',

      packages=find_packages(),
      install_requires=[
            'pandas==2.2.3',
            'beautifulsoup4==4.12.3',
            'requests==2.31.0',
            'aiohttp==3.9.5',
            'aiofiles==24.1.0'
      ],
      include_package_data=True,
      zip_safe=False)



import json
import pkgutil

# Load the JSON data from the file
def load_wormbase_api_json(call_type, call_class):
    data_bytes = pkgutil.get_data(__name__, f"data/wormbase_{call_type}_{call_class}.json")
    data_str = data_bytes.decode('utf-8')
    return json.loads(data_str)
